// ----------------------------------------------------------------------------
// Copyright (C) 2003-2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples
{
    using System;
    using System.Configuration;
    using System.Collections;
    using System.Globalization;
    using System.ServiceModel;
    using System.ServiceModel.Configuration;

    public class SampleProfileUdpBindingConfigurationElement : StandardBindingConfigurationElement
    {
        public SampleProfileUdpBindingConfigurationElement(string configurationName)
            : base(configurationName)
        {
        }

        public SampleProfileUdpBindingConfigurationElement()
            : this(null)
        {
        }

        [ConfigurationProperty(UdpConfigurationStrings.OrderedSession, DefaultValue = UdpDefaults.OrderedSession)]
        public bool OrderedSession
        {
            get { return (bool)base[UdpConfigurationStrings.OrderedSession]; }
            set { base[UdpConfigurationStrings.OrderedSession] = value; }
        }

        [ConfigurationProperty(UdpConfigurationStrings.ReliableSessionEnabled, DefaultValue = UdpDefaults.ReliableSessionEnabled)]
        public bool ReliableSessionEnabled
        {
            get { return (bool)base[UdpConfigurationStrings.ReliableSessionEnabled]; }
            set { base[UdpConfigurationStrings.ReliableSessionEnabled] = value; }
        }

        [ConfigurationProperty(UdpConfigurationStrings.SessionInactivityTimeout, DefaultValue = UdpDefaults.SessionInactivityTimeoutString)]
        [TimeSpanValidator(MinValueString = "00:00:00")]
        public TimeSpan SessionInactivityTimeout
        {
            get { return (TimeSpan)base[UdpConfigurationStrings.SessionInactivityTimeout]; }
            set { base[UdpConfigurationStrings.SessionInactivityTimeout] = value; }
        }

        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                ConfigurationPropertyCollection properties = base.Properties;
                properties.Add(new ConfigurationProperty("orderedSession", typeof(System.Boolean), false, null, null, System.Configuration.ConfigurationPropertyOptions.None));
                properties.Add(new ConfigurationProperty("reliableSessionEnabled", typeof(System.Boolean), true, null, null, System.Configuration.ConfigurationPropertyOptions.None));
                properties.Add(new ConfigurationProperty("sessionInactivityTimeout", typeof(System.TimeSpan), System.TimeSpan.Parse("00:10:00"), null, new System.Configuration.TimeSpanValidator(System.TimeSpan.Parse("00:00:00"), System.TimeSpan.Parse("10675199.02:48:05.4775807"), false), System.Configuration.ConfigurationPropertyOptions.None));
                return properties;
            }
        }        
        
        protected override void InitializeFrom(Binding binding)
        {
            if (binding == null)
                throw new ArgumentNullException("binding");

            if (binding.GetType() != typeof(SampleProfileUdpBinding))
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentCulture,
                    "Invalid type for binding. Expected type: {0}. Type passed in: {1}.",
                    typeof(SampleProfileUdpBinding).AssemblyQualifiedName,
                    binding.GetType().AssemblyQualifiedName));
            }
            SampleProfileUdpBinding udpBinding = (SampleProfileUdpBinding)binding;

            this.OrderedSession = udpBinding.OrderedSession;
            this.ReliableSessionEnabled = udpBinding.ReliableSessionEnabled;
            this.SessionInactivityTimeout = udpBinding.SessionInactivityTimeout;
        }

        protected override void OnApplyConfiguration(Binding binding)
        {
            if (binding == null)
                throw new ArgumentNullException("binding");

            if (binding.GetType() != typeof(SampleProfileUdpBinding))
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentCulture,
                    "Invalid type for binding. Expected type: {0}. Type passed in: {1}.",
                    typeof(SampleProfileUdpBinding).AssemblyQualifiedName,
                    binding.GetType().AssemblyQualifiedName));
            }
            SampleProfileUdpBinding udpBinding = (SampleProfileUdpBinding)binding;

            udpBinding.OrderedSession = this.OrderedSession;
            udpBinding.ReliableSessionEnabled = this.ReliableSessionEnabled;
            udpBinding.SessionInactivityTimeout = this.SessionInactivityTimeout;
        }
    }
}
