// ----------------------------------------------------------------------------
// Copyright (C) 2003-2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples
{
    using System;
    using System.Net.Sockets;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.Globalization;

    /// <summary>
    /// Collection of constants used by the Udp Channel classes
    /// </summary>
    static class UdpConstants
    {
        static MessageEncoderFactory messageEncoderFactory;
        internal const string EventLogSourceName = "Microsoft.ServiceModel.Samples";
        internal const int MaxBufferSize = 64 * 1024;
        internal const string Scheme = "soap.udp";
        internal const string UdpBindingSectionName = "system.serviceModel/bindings/sampleProfileUdpBinding";
        internal const string UdpTransportSectionName = "udpTransport";
        internal const int WSAETIMEDOUT = 10060;

        static UdpConstants()
        {
            messageEncoderFactory = new TextMessageEncoderFactory();
        }

        // ensure our advertised MessageVersion matches the version we're
        // using to serialize/deserialize data to/from the wire
        internal static MessageVersion MessageVersion
        {
            get
            {
                return messageEncoderFactory.MessageVersion;
            }
        }

        // we can use the same encoder for all our Udp Channels as it's free-threaded
        internal static MessageEncoder MessageEncoder
        {
            get
            {
                return messageEncoderFactory.Encoder;
            }
        }
    }

    static class UdpConfigurationStrings
    {
        internal const string OrderedSession = "orderedSession";
        internal const string ReliableSessionEnabled = "reliableSessionEnabled";
        internal const string SessionInactivityTimeout = "sessionInactivityTimeout";
    }

    static class UdpChannelHelpers
    {
        static MessageEncoderFactory messageEncoderFactory = new TextMessageEncoderFactory();

        /// <summary>
        /// The Channel layer normalizes exceptions thrown by the underlying networking implementations
        /// into subclasses of ChannelException, so that Channels can be used polymorphically from
        /// an exception handling perspective.
        /// </summary>
        internal static ChannelException ConvertTransferException(SocketException socketException)
        {
            return new ChannelIOException(
                string.Format(CultureInfo.CurrentCulture, 
                "A Udp error ({0}: {1}) occurred while transmitting data.", socketException.ErrorCode, socketException.Message), 
                socketException);
        }
    }

    static class UdpDefaults
    {
        internal const bool OrderedSession = true;
        internal const bool ReliableSessionEnabled = true;
        internal const string SessionInactivityTimeoutString = "00:10:00";
    }
}
