// ----------------------------------------------------------------------------
// Copyright (C) 2003-2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace Microsoft.ServiceModel.Samples
{
    class UdpInputChannel : ChannelBase, IInputChannel
    {
        EndpointAddress localAddress;
        InputQueue<Message> messageQueue;

        internal UdpInputChannel(UdpListenerFactory factory, EndpointAddress localAddress)
            : base(factory)
        {
            this.localAddress = localAddress;
            this.messageQueue = new InputQueue<Message>();
        }

        public EndpointAddress LocalAddress
        {
            get
            {
                return this.localAddress;
            }
        }


        public int PendingMessageCount
        {
            get
            {
                return this.messageQueue.PendingCount;
            }
        }

        //Starts an asynchronous receive operation which will
        //terminate when a message arrives on this channel.
        public IAsyncResult BeginReceive(AsyncCallback callback, object state)
        {
            ThrowIfDisposedOrNotOpen();
            return this.messageQueue.BeginDequeue(callback, state);
        }


        public Message EndReceive(IAsyncResult result)
        {
            return this.messageQueue.EndDequeue(result);
        }

        //Hands the message off to other components higher up the
        //channel stack that have previously called BeginReceive() 
        //and are waiting for messages to arrive on this channel.
        internal void Dispatch(Message message)
        {
            this.messageQueue.EnqueueAndDispatch(message);
        }

        //Closes the channel ungracefully during error conditions.
        protected override void OnAbort()
        {
            try
            {
                this.messageQueue.Close();
            }
            finally
            {
                base.OnAbort();
            }
        }

        //Closes the channel gracefully during normal conditions.
        protected override void OnClose()
        {
            this.messageQueue.Close();
            base.OnClose();
        }

        //Opens the channel and allows it to receive messages.
        protected override void OnOpen()
        {
            base.OnOpen();
            this.messageQueue.Open();
        }

        //Initiates a synchronous Receive operation and blocks until
        //a message arrives on this channel.
        public Message Receive()
        {
            ThrowIfDisposedOrNotOpen();
            return this.messageQueue.Dequeue();
        }
    }
}